# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : user_dao.py 
# @Software: Visual Studio Code

from sqlalchemy.dialects.postgresql import insert
from sqlalchemy import select

from dao import BaseDao
from dao.model import User


class UserDao(BaseDao):

    model = User


    @classmethod
    async def insert_not_exist(cls, data: model):
        stmt = (
            insert(cls.model).values(user_id=data.user_id, 
                                     user_name=data.user_name, 
                                     user_ref=data.user_ref, 
                                     lang_code=data.lang_code,
                                     referral_code=data.referral_code,
                                     referraled_code=data.referraled_code).on_conflict_do_nothing(
                index_elements=['user_id']
            )
        )
        async with cls.session() as session:
            await session.execute(stmt)

    @classmethod
    async def get_by_user_id(cls, user_id: str) -> User:
        async with cls.session() as session:
            query = select(cls.model).filter_by(user_id=user_id)
            result = await session.execute(query)
            user = result.scalars().first()
            session.expunge(user)
            return user
    
    @classmethod
    async def get_by_referral_code(cls, code: str) -> User:
        async with cls.session() as session:
            query = select(cls.model).filter_by(referral_code=code)
            result = await session.execute(query)
            user = result.scalars().first()
            session.expunge(user)
            return user